/**
 * Copyright (c) 2010 jolira. All rights reserved. This program and the accompanying materials are made available under
 * the terms of the GNU Public License 2.0 which is available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 */

package com.google.code.joliratools.logdb.parser;

import java.io.InputStream;

/**
 * A parser for processing delimited files.
 * 
 * @author jfk
 * @date Sep 13, 2010 11:09:15 PM
 * @since 1.0
 */
public class DelimitedParserFactory implements ParserFactory {
    private static final long serialVersionUID = -743708559515638117L;
    private final String fieldDelimiter;
    private final int timeStampPos;
    private final String timeStampFormat;
    private final int classPos;
    private final int methodPos;
    private final int levelPos;
    private final int maxFieldPos;

    /**
     * @param fieldDelimiter
     *            the delimiters to be used to break the records into fields
     * @param timeStampPos
     *            the position of the timestamp field
     * @param timeStampFormat
     *            the format of the timestamp field
     * @param classPos
     *            the position of the class name
     * @param methodPos
     *            the position of the method name
     * @param levelPos
     *            the position of the field containing the level
     * @param maxFieldPos
     *            the maximum field position (stop searching for delimiters once this pos is reached)
     */
    public DelimitedParserFactory(final String fieldDelimiter, final int timeStampPos, final String timeStampFormat,
            final int classPos, final int methodPos, final int levelPos, final int maxFieldPos) {
        this.fieldDelimiter = fieldDelimiter;
        this.timeStampFormat = timeStampFormat;
        this.timeStampPos = timeStampPos;
        this.classPos = classPos;
        this.methodPos = methodPos;
        this.levelPos = levelPos;
        this.maxFieldPos = maxFieldPos;
    }

    /**
     * Create a new parser. Allows subclasses to override the parser to be used.
     * 
     * @param in
     *            the input stream
     * @param fldDelim
     * @param timeStampFmt
     * @param timeStmpPos
     * @param clsPos
     * @param methdPos
     * @param lvlPos
     * @param mxFldPos
     * @return the parser
     */
    protected Parser createParser(final InputStream in, final String fldDelim, final String timeStampFmt,
            final int timeStmpPos, final int clsPos, final int methdPos, final int lvlPos, final int mxFldPos) {
        return new DelimitedParser(in, fldDelim, timeStampFmt, timeStmpPos, clsPos, methdPos, lvlPos, mxFldPos);
    }

    @Override
    public Parser parse(final InputStream in) {
        return createParser(in, fieldDelimiter, timeStampFormat, timeStampPos, classPos, methodPos, levelPos,
                maxFieldPos);
    }
}
