/**
 * Copyright (c) 2010 jolira. All rights reserved. This program and the accompanying materials are made available under
 * the terms of the GNU Public License 2.0 which is available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 */
package com.google.code.joliratools.logdb.parser;

import java.util.Date;
import java.util.List;

/**
 * The record.
 * 
 * @author jfk
 * @date Oct 23, 2010 5:06:22 PM
 * @since 1.0
 */
public class DelimitedRecord implements Record {
    private final String level;
    private final List<String> variables;
    private final long timestamp;
    private final String clazz;
    private final String method;
    private final int lineNo;
    private final StringBuilder raw;
    private final Type type;

    /**
     * Create a new record.
     * 
     * @param type
     * @param lineNo
     * @param raw
     * @param variables
     * @param timestamp
     * @param clazz
     * @param method
     * @param level
     */
    public DelimitedRecord(final Type type, final int lineNo, final String raw, final List<String> variables,
            final long timestamp, final String clazz, final String method, final String level) {
        this.type = type;
        this.lineNo = lineNo;
        this.raw = new StringBuilder();
        this.variables = variables;
        this.timestamp = timestamp;
        this.clazz = clazz;
        this.method = method;
        this.level = level;

        this.raw.append(raw);
    }

    /**
     * Add a line to the existing record.
     * 
     * @param line
     *            the line
     */
    public void append(final String line) {
        raw.append(DelimitedParser.NEWLINE);
        raw.append(line);

        final int size = variables.size();
        final StringBuilder buf = new StringBuilder();
        final int last = size - 1;

        buf.append(variables.get(last));
        buf.append(DelimitedParser.NEWLINE);
        buf.append(line);

        variables.set(last, buf.toString());
    }

    @Override
    public String getClazz() {
        return clazz;
    }

    @Override
    public String getId() {
        return "" + clazz + '#' + method + '[' + timestamp + ']';
    }

    @Override
    public String getLevel() {
        return level;
    }

    @Override
    public int getLineNumber() {
        return lineNo;
    }

    @Override
    public String getMethod() {
        return method;
    }

    @Override
    public String getRaw() {
        return raw.toString();
    }

    @Override
    public long getTimestamp() {
        return timestamp;
    }

    @Override
    public Type getType() {
        return type;
    }

    @Override
    public String[] getVariables() {
        final int size = variables.size();

        return variables.toArray(new String[size]);
    }

    @Override
    public String toString() {
        final StringBuilder builder = new StringBuilder();

        builder.append("DelimitedRecord [lineNo=");
        builder.append(lineNo);

        if (type != null) {
            builder.append(", type=");
            builder.append(type);
        }

        builder.append(", timestamp=");
        builder.append(new Date(timestamp));

        if (clazz != null) {
            builder.append(", clazz=");
            builder.append(clazz);
        }

        if (method != null) {
            builder.append(", method=");
            builder.append(method);
        }

        if (level != null) {
            builder.append(", level=");
            builder.append(level);
        }

        if (variables != null) {
            builder.append(", variables=");
            builder.append(variables);
        }

        if (raw != null) {
            builder.append(", raw=");
            builder.append(raw);
        }

        builder.append("]");
        return builder.toString();
    }
}
